; DV3 Q68 SDHC Card Read Sector  1.00	 2016-2017 W. Lenerz

	section dv3

	xdef	hd_rdirect		; direct read sector
	xdef	hd_rsint		; internal read sector


	xref	hd_hold 		; reserve the interface
	xref	hd_release		; release the interface
	xref	snd_cmd 		; send a command to sdhc

	include 'dev8_keys_java'
	include 'DEV8_dv3_keys'
	include 'DEV1_q68_qlsd_keys'
	include 'DEV8_keys_err'
	include 'DEV8_dv3_hd_keys'

readreg reg	d1/d2/d3/d7/a1/a2/a3

;+++
; This routine reads a sector from an sdhc card.  The sector number is that of
; the absolute sector on the SDHC card.
; All normal read accesses to the card go through here.
; Read is done via background transfer.
;
;	d0 cr	absolute sector (=block) number  / error code
;	d1    p
;	d2 c  p number of sectors to read (1 = 1 sector, 2 = 2 sects etc)
;	d7 c  p drive ID / number  (from 1 to 8)
;	a1 c  p address to read into , MUST be large enough to hold the total nbr of sectors requested
;	a3 c  p linkage block
;	a4 c  p drive definition
;
;
;	status return 0 or ERR.MCHK
;	The sector number is for sectors 512 bytes long. The QLSD interface needs
;	a sector (block) position starting from the very first sector.
;---
hdr_inus
	rts
hd_rdirect
hd_rsint
	jsr	hd_hold 		; hold the drive for me
	bne.s	hdr_inus		; can't
	movem.l readreg,-(a7)

	move.l	#IF_BASE,a2
	tst.b	SPI_SELECT1(a2) 	; select the card

	add.l	ddf_dtop-4(a4),d0	; !!!!!!!!!

	move.l	d0,d7			; sector to get
	subq.l	#1,d2			; go around d2 times
; first send command16 ($51) to the card, to determine block to be read
; ($51, 4 bytes with the address to read from, $ff)
readloop
	move.l	d7,d0			; sector nbr to read (0a0b0c0d)
	moveq	#-1,d1			; ffffffff
	move.b	d0,d1			; ffffff0d
	rol.l	#8,d1			; ffff0dff : d1.w = last byte of address , $ff
	move.b	#$51,d0 		; 0a0b0c51
	ror.l	#8,d0			; 510a0b0c : $51 and first 3 bytes of address
	bsr	snd_cmd 		; send command to card
	bne.s	hdr_rts 		; ooops
	tst.b	d1			; return must be 0
	bne.s	hdr_err 		; ooops again

; wait for reply from card **** possible total blockage if card never replies correctly

	move.l	#SPI_BG_READ,a3
waitlp1 tst.b	(a3)			; prep for transfer
	move.b	SPI_READ(a2),d0 	; get byte
	cmp.b	#$fe,d0 		; was it $fe (would addq #2,d0 be faster?
	bne.s	waitlp1 		; no, wait until you get this back

; got a correct answer back from card, it's ready for transfer
; now read 512 times one byte

	move.w	#511,d3 		; read 512 bytes
getlp	tst.b	(a3)			; prep for transfer
	move.b	SPI_READ(a2),d1
	move.b	d1,(a1)+		; get byte
	dbf	d3,getlp

; get rid of the 2 CRCs sent
	tst.b	(a3)			; prep for transfer
	tst.b	SPI_READ(a2)
	tst.b	(a3)			; prep for transfer
	tst.b	SPI_READ(a2)
	addq.l	#1,d7			; get possible next block
rd_dbf	dbf	d2,readloop

hdr_ok	moveq	#0,d0
hdr_rts tst.b	SPI_SELECT0(a2) 	; de-select the card
	tst.l	d0
	movem.l (a7)+,readreg
	jmp	hd_release		; will jump to hd_release & test d0
hdr_err
	moveq	#err.mchk,d0		; signal drive wasn't found
	bra.s	hdr_rts

	end
